@@ -1,18 +1,22 @@
+from typing import TypeVar
+
 import regex as re
 
 from ..core import ChepyCore, ChepyDecorators
 
+LinksT = TypeVar("LinksT", bound="Links")
+
 
 class Links(ChepyCore):
     def __init__(self, *data):
         super().__init__(*data)
 
     @ChepyDecorators.call_stack
-    def pastebin_to_raw(self):
+    def pastebin_to_raw(self) -> LinksT:
         """Convert a pastebin link to raw pastebin link
-        
+
         Returns:
-            Chepy: The Chepy object. 
+            Chepy: The Chepy object.
 
         Examples:
             >>> Chepy("https://pastebin.com/abCD").pastebin_to_raw()
@@ -22,11 +26,11 @@ class Links(ChepyCore):
         return self
 
     @ChepyDecorators.call_stack
-    def github_to_raw(self):
+    def github_to_raw(self) -> LinksT:
         """Convert a github link to raw github link
-        
+
         Returns:
-            Chepy: The Chepy object. 
+            Chepy: The Chepy object.
 
         Examples:
             >>> Chepy("https://github.com/securisec/chepy/blob/master/README.md").github_to_raw()
